
// ************************ BEGIN_COPYRIGHT_NOTICE ************************
//
// This source code copyright (c) 1995 by Eric Dill and Paul Keister
//
// Permission to use, copy, modify, distribute, and sell this software
// and its documentation for any purpose is hereby granted without
// fee, provided that this copyright notice appear in all copies
// and that the author's name not be used in advertising or publicity
// pertaining to distribution of the software without specific, written
// prior permission.  The author makes no representations about the
// suitability of this software for any purpose.  It is provided "as
// is" without express or implied warranty.
//
// Please send all bug fixes, ideas, and comments to:
//
//             Via Email to CompuServe 75061,3310 and 71242,1145
//             Or post to the Borland OWL2:Print/Doc/View Forum on CompuServe
//
// ************************* END_COPYRIGHT_NOTICE *************************

//
// base class required to fit special effects into stream operator semanics
//

#ifndef __PRTMANIP__H
#define __PRTMANIP__H


class SpecialEffectManipBase
{
   public:
      enum { bufSize = 32 };
      SpecialEffectManipBase(     void )           {}
      virtual void WriteHeader(   ostream& out ) = 0;
      virtual void WriteData(     ostream& out ) = 0; // Don't implement trailing WHITESP
      friend ostream& operator<<( ostream& out, SpecialEffectManipBase& mb );

   protected:
      static char buf[ bufSize ];   // for istream float bug under Win-95
};


inline ostream& operator<<( ostream& out, SpecialEffectManipBase& mb )
{
   long savedFlags  = out.flags();  // save format state
   out.flags( 0L );                 // set default format state
   mb.WriteHeader( out );           // write manipulator header
   mb.WriteData(   out );           // write manipulator data
   out << ESC;                      // write manipulator terminator
   out.flags( savedFlags );         // restore format state
   return out;
}


//
// template for special effects object interface
//

template< SpecialEffect eff >
class SpecialEffectManip : public SpecialEffectManipBase
{
   protected:
      virtual void WriteHeader( ostream& out )
         {
            //
            // each object sets its own handler when writing to a stream
            //

            dynamic_cast< prtStream& >( out ).RenderFunctions[ eff ]
               =  SpecialEffectManip< eff >::Render;

            out << ESC << ESC << eff << WHITESP;
         }


      // ********************** IMPORTANT **********************
      //
      // The Render functions _must_ extract from the istream&
      // that they recieve: the trailing WHITESP written by
      // WriteHeader() _and_ all characters written by the
      // virtual WiteData() member function
      //
      // *******************************************************

      static void Render( prtStream *pstream, istream& in );

};


// ****************************************************************
// *                                                              *
// * SpecialEffect Manipulators                                   *
// *                                                              *
// *   Note: Don't implement trailing WHITESP in ...::WriteData() *
// *                                                              *
// ****************************************************************

//
// example of a special effects manipulator object
//

class SingleBox : public SpecialEffectManip< Box >
{
   public:
      SingleBox(  float top, float bottom, float left, float right )
               :  Top( top ), Bottom( bottom ), Left( left ), Right( right )  {}

      virtual void WriteData( ostream& out )
         {
            out << Top     << WHITESP
                << Bottom  << WHITESP
                << Left    << WHITESP
                << Right   << WHITESP;
         }

   private:
      float Top, Bottom, Left, Right;
};


//
// a special effect manipulator to draw a frame
//

class drawFrame : public SpecialEffectManip< Frame >
{
   public:
      drawFrame(  float top,        float left,
                  float width,      float height,
                  float lineWidth = 0.001,
                  int   lineStyle = PS_SOLID,
                  int   red       = 0,
                  int   green     = 0,
                  int   blue      = 0  ) 
               :  Top(        top       ),   Left(       left      ),
                  Width(      width     ),   Height(     height    ),
                  LineStyle(  lineStyle ),   LineWidth(  lineWidth ),
                  Red(        red       ),
                  Green(      green     ),
                  Blue(       blue      )          {}

      virtual void WriteData( ostream& out )
         {
            out << Top       << WHITESP   << Left      << WHITESP
                << Width     << WHITESP   << Height    << WHITESP 
                << LineWidth << WHITESP   << LineStyle << WHITESP
                << Red       << WHITESP
                << Green     << WHITESP   << Blue;
         }

   private:
      float Top, Left, Width, Height, LineWidth;
      int   LineStyle, Red,   Green,  Blue;
};


//
// a special effect manipulator for bitmap resource id's
//

class bitMap : public SpecialEffectManip< BitMap >
{
   public:
      bitMap( TResId resId, float x=0.0, float y=0.0, float w=0.0, float h=0.0 )
         : ResName( resId ), xPos( x ), yPos( y ), width(w), height(h) {}

      virtual void WriteData( ostream& out )
         {
            out << xPos  << WHITESP << yPos   << WHITESP
                << width << WHITESP << height << WHITESP << ResName << ends;
         }

   private:
      float xPos, yPos, width, height;
      char *ResName;
};


//
// a special effect manipulator to draw a line
//                                     contributed by Steven Carr
//

class drawLine : public SpecialEffectManip< Line >
{
   public:
      drawLine(   float top,        float left,
                  float width,      float height,
                  float lineWidth = 0.001,
                  int   lineStyle = PS_SOLID,
                  int   red       = 0,
                  int   green     = 0,
                  int   blue      = 0  )
               :  Top(        top       ),   Left(       left      ),
                  Width(      width     ),   Height(     height    ),
                  LineStyle(  lineStyle ),   LineWidth(  lineWidth ),
                  Red(        red       ),
                  Green(      green     ),
                  Blue(       blue      )          {}

      virtual void WriteData( ostream& out )
         {
            out   << Top       << WHITESP   << Left      << WHITESP
                  << Width     << WHITESP   << Height    << WHITESP
                  << LineWidth << WHITESP   << LineStyle << WHITESP
                  << Red       << WHITESP
                  << Green     << WHITESP   << Blue;
         }

   private:
      float Top, Left, Width, Height, LineWidth;
      int   LineStyle, Red,   Green,  Blue;
};


//
// a special effect manipulator for setting the font
//

class setFont : public SpecialEffectManip< Font >
{
   public:
      setFont( TFont& font )
         {
            font.GetObject( lf );
         }

      virtual void WriteData( ostream& out )
         {
            out << lf.lfHeight         << WHITESP
                << lf.lfWidth          << WHITESP
                << lf.lfEscapement     << WHITESP
                << lf.lfOrientation    << WHITESP
                << lf.lfWeight         << WHITESP
                << lf.lfItalic         << WHITESP
                << lf.lfUnderline      << WHITESP
                << lf.lfStrikeOut      << WHITESP
                << lf.lfCharSet        << WHITESP
                << lf.lfOutPrecision   << WHITESP
                << lf.lfClipPrecision  << WHITESP
                << lf.lfQuality        << WHITESP
                << lf.lfPitchAndFamily << WHITESP
                << lf.lfFaceName       << ends;
         }

   private:
      LOGFONT  lf;
};


//
// a special effect manipulator to set the left margin
//

class setLeftMargin : public SpecialEffectManip< LeftMargin >
{
   public:
      setLeftMargin( float m = defLeftM )
         : leftMargin( m )                         {}

      virtual void WriteData( ostream& out )
         {
            out << leftMargin << WHITESP;
         }

   private:
      float leftMargin;
};


//
// a special effect manipulator to set the top margin
//

class setTopMargin  : public SpecialEffectManip< TopMargin >
{
   public:
      setTopMargin( float m = defTopM )
         : topMargin( m )                          {}

      virtual void WriteData( ostream& out )
         {
            out << topMargin << WHITESP;
         }

   private:
      float topMargin;
};


//
// a special effect manipulator to set the bottom margin
//

class setBottomMargin  : public SpecialEffectManip< BottomMargin >
{
   public:
      setBottomMargin( float m = defBotM )
         : botMargin( m )                          {}

      virtual void WriteData( ostream& out )
         {
            out << botMargin << WHITESP;
         }

   private:
      float botMargin;
};


//
// a special effect manipulator to set the print position
//
//    note: this manupilator takes y,x (NOT x,y)

class setPrintPos : public SpecialEffectManip< SetPrintPos >
{
   public:
      setPrintPos( float y, float x = -1, bool m = false )
         : yPos( y ), xPos( x ), noMargins( m )    {}

      virtual void WriteData( ostream& out )
         {
            out << yPos << WHITESP << xPos << WHITESP << noMargins;
         }

   private:
      float yPos,
            xPos;
      bool  noMargins;
};


//
// a special effect manipulator to set the print column position
//                                     contributed by Steven Carr
//
//    note: this manupilator takes x (NOT x,y OR y,x )

class setPrintCol : public SpecialEffectManip< SetPrintCol >
{
   public:
      setPrintCol( float x = -1, bool m = false )
         : xPos( x ), noMargins( m )    {}

      virtual void WriteData( ostream& out )
         {
            out << xPos << WHITESP << noMargins;
         }

   private:
      float xPos;
      bool  noMargins;
};


//
// a special effect manipulator to set the tabstop interval
//

class setTabInterval : public SpecialEffectManip< TabInterval >
{
   public:
      setTabInterval( float t = 0.50 )
         : tabInterval ( t )                       {}

      virtual void WriteData( ostream& out )
         {
            out << tabInterval << WHITESP;
         }

   private:
      float tabInterval;
};


//
// a special effect manipulator to save print positions
//

class saveState : public SpecialEffectManip< SaveState >
{
   public:
      saveState( prtStreamState state = PrintPos_, int i = 0 )
         : State( state ), Index( i )              {}

      virtual void WriteData( ostream& out )
         {
            out << State   << WHITESP  << Index;
         }

   private:
      prtStreamState State;
      int            Index;
};


//
// a special effect manipulator to restore saved print positions
//

class restoreState : public SpecialEffectManip< RestoreState >
{
   public:
      restoreState( prtStreamState state = PrintPos_, int i = 0 )
         : State( state ), Index( i )              {}

      virtual void WriteData( ostream& out )
         {
            out << State   << WHITESP  << Index;
         }

   private:
      prtStreamState State;
      int            Index;
};


//
// a special effect manipulator to force a page break
//

class endpg : public SpecialEffectManip< PageBreak >
{
   public:
      endpg()                                      {}

      virtual void WriteData( ostream& out )       
      {
         prtStream *prt = dynamic_cast<prtStream *>(&out);
         prt->NextPage();
      }
};


//
// a special effect manipulator for conditional page breaks
//

class conditionalPageBreak : public SpecialEffectManip< CondPageBreak >
{
   public:
      conditionalPageBreak( float height= 0, float limit = defPrtLimit )
         : printSize( height ), pageLimit( limit ) {}

      virtual void WriteData( ostream& out )
         {
            out << printSize  << WHITESP  << pageLimit << WHITESP;
         }

   private:
      float printSize,
            pageLimit;
};


//
// a special effect manipulator to print the current page number
//

class pageNumber : public SpecialEffectManip< PrtPageNum >
{
   public:
      pageNumber( void )                           {}

      virtual void WriteData( ostream& out )       {}
};


//
// a special effect manipulator to print the number of pages
//

class numPages : public SpecialEffectManip< PrtNumPages >
{
   public:
      numPages( void )                             {}

      virtual void WriteData( ostream& out )       {}
};


//
// a special effect manipulator for page headers
//

class startHeader : public SpecialEffectManip< StartHeader >
{
   public:
      startHeader( void )                          {}
      virtual void WriteData( ostream& out )       {}
};

class endHeader : public SpecialEffectManip< EndHeader >
{
   public:
      endHeader( void )                            {}
      virtual void WriteData( ostream& out )       {}
};


//
// a special effect manipulator for page footers
//

class startFooter : public SpecialEffectManip< StartFooter >
{
   public:
      startFooter( void )                          {}
      virtual void WriteData( ostream& out )       {}
};

class endFooter : public SpecialEffectManip< EndFooter >
{
   public:
      endFooter( void )                            {}
      virtual void WriteData( ostream& out )       {}
};

#endif //  __PRTNAMIP__H
